@@ -186,6 +186,8 @@ End:
 
 #ifdef HEADER
 
+//#define VERBOSE 1
+
 /* Alphabet over unicode characters. */
 
 typedef struct ualphabet {
@@ -289,6 +291,17 @@ typedef struct text_fuzzy {
 
     candidate_t first;
     candidate_t * last;
+
+    /* When scanning an array, put the index of the element of the
+       array into "text_fuzzy->offset". The offset of the nearest
+       elements are preserved in the "candidate_t" linked list which
+       starts off with "text_fuzzy". 
+
+       There is currently no sanity check, so if the user forgets to
+       set "offset" each time around the loop, the code will not
+       notice anything amiss and just send a list of zeros back to the
+       user. */
+
     int offset;
 
     /* Does the user want to use an alphabet filter? Default is yes,
@@ -324,6 +337,11 @@ typedef struct text_fuzzy {
 }
 text_fuzzy_t;
 
+/* The string is not unicode so its length in unicode characters is
+   unknown. */
+
+#define TEXT_FUZZY_INVALID_UNICODE_LENGTH -1
+
 #endif /* HEADER */
 
 /* The following calculations need to be done twice, first when
@@ -496,7 +514,7 @@ static int ualphabet_miss (text_fuzzy_t * tf, text_fuzzy_string_t * b)
 
 	/* If we have too many misses, stop searching. */
 
-	if (misses >= tf->max_distance) {
+	if (misses > tf->max_distance) {
 	    MESSAGE ("%s:%s: %d misses over %d: ",
 		    tf->text.text, tf->b.text, misses, tf->max_distance);
 	    return 1;
@@ -828,6 +846,12 @@ FUNC (begin_scanning) (text_fuzzy_t * text_fuzzy)
     text_fuzzy->alphabet_rejections = 0;
     text_fuzzy->length_rejections = 0;
 
+    /* Set up the linked list. */
+
+    if (text_fuzzy->wantarray) {
+	text_fuzzy->last = & text_fuzzy->first;
+    }
+
     OK;
 }
 
@@ -1000,6 +1024,75 @@ FUNC (free_memory) (text_fuzzy_t * text_fuzzy)
     OK;
 }
 
+FUNC (set_max_distance) (text_fuzzy_t * text_fuzzy, int max_distance)
+{
+    text_fuzzy->max_distance = max_distance;
+    OK;
+}
+
+FUNC (get_max_distance) (text_fuzzy_t * text_fuzzy, int * max_distance)
+{
+    * max_distance = text_fuzzy->max_distance;
+    OK;
+}
+
+FUNC (set_transpositions) (text_fuzzy_t * text_fuzzy, int transpositions)
+{
+    text_fuzzy->transpositions_ok = transpositions != 0 ? 1 : 0;
+    OK;
+}
+
+FUNC (get_transpositions) (text_fuzzy_t * text_fuzzy, int * transpositions)
+{
+    * transpositions = text_fuzzy->transpositions_ok;
+    OK;
+}
+
+FUNC (last_distance) (text_fuzzy_t * text_fuzzy, int * last_distance)
+{
+    * last_distance = text_fuzzy->distance;
+    OK;
+}
+
+FUNC (no_alphabet) (text_fuzzy_t * text_fuzzy, int yes_no)
+{
+    text_fuzzy->user_no_alphabet = yes_no != 0 ? 1 : 0;
+    if (text_fuzzy->user_no_alphabet) {
+	text_fuzzy->use_alphabet = 0;
+	text_fuzzy->use_ualphabet = 0;
+    }
+    OK;
+}
+
+FUNC (ualphabet_rejections) (text_fuzzy_t * text_fuzzy, int * ualphabet_rejections)
+{
+    * ualphabet_rejections = text_fuzzy->ualphabet.rejections;
+    OK;
+}
+
+FUNC (set_no_exact) (text_fuzzy_t * text_fuzzy, int yes_no)
+{
+    text_fuzzy->no_exact = yes_no != 0 ? 1 : 0;
+    OK;
+}
+
+FUNC (get_length_rejections) (text_fuzzy_t * text_fuzzy, int * length_rejections)
+{
+    * length_rejections = text_fuzzy->length_rejections;
+    OK;
+}
+
+FUNC (get_unicode_length) (text_fuzzy_t * text_fuzzy, int * unicode_length)
+{
+    if (text_fuzzy->text.unicode) {
+	* unicode_length = text_fuzzy->text.ulength;
+    }
+    else {
+	* unicode_length = TEXT_FUZZY_INVALID_UNICODE_LENGTH;
+    }
+    OK;
+}
+
 /* statuses:
 
 status: open_error
